package at.xaxa.ledger.ui.add

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.lifecycle.viewmodel.compose.viewModel
import at.xaxa.ledger.data.Entry
import at.xaxa.ledger.data.EntryRepository
import at.xaxa.ledger.data.db.Category.CategoryEntity
import at.xaxa.ledger.data.db.Entry.EntryEntity
import at.xaxa.ledger.ui.home.EntryListUIState
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.flowOf
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

data class CategoryListUIState(val entry: Flow<List<CategoryEntity>> = flowOf(emptyList()))

var categoryUiState by mutableStateOf(CategoryListUIState())
    private set

// UI State to hold a list of games
data class EntryUi(
    val entries: Flow<List<Entry>> = flow { emit(emptyList()) }
)

class AddViewModel(
    private val savedStateHandle: SavedStateHandle,
    private val repository: EntryRepository
) : ViewModel() {
    var addUiState by mutableStateOf(EntryUi())

    fun addEntryToDB(entry: Entry) {
        viewModelScope.launch {
            try {
                val entryEntity = Entry(
                    id = entry.id,
                    name = entry.name,
                    date = entry.date,
                    amount = entry.amount,
                    categoryID = entry.categoryID
                )

                withContext(Dispatchers.IO) {
                    repository.insertEntry(entryEntity) // Add game to the database
                }
                // Optionally, log or update UI state to reflect that the game was added
            } catch (e: Exception) {
            }
        }
    }

    fun addCategory(category: CategoryEntity) {
        viewModelScope.launch {
            try {
                val categoryEntity = CategoryEntity(
                    _id = category._id,
                    categoryName = category.categoryName,
                    icon = category.icon,
                )

                withContext(Dispatchers.IO) {
                    repository.insertCategory(categoryEntity) // Add game to the database
                }
                // Optionally, log or update UI state to reflect that the game was added
            } catch (e: Exception) {
            }
        }
    }

    fun getAllCategories() {
        viewModelScope.launch {
            val categories = withContext(Dispatchers.IO) {
                repository.getAllCategories()
            }
            categoryUiState = CategoryListUIState(categories)

        }

    }
}
